<?php
// +----------------------------------------------------------------------+
// | Page generation class.                                               |
// +----------------------------------------------------------------------+
// | This object handles generating the page - the caller just needs to   |
// | include this one and then instantiate the class. Then, just contain  |
// | HTML.                                                                |
// |                                                                      |
// | Instantiating the class buffers the output. Once page ends, call the |
// | display method to output.                                            |
// +----------------------------------------------------------------------+
// | @version $Revision: 1.14 $
// +----------------------------------------------------------------------+

define("TMPL_MAIN","site.tmpl");
define("DEFAULT_TITLE",HTML_TITLE);
define("TIMER",true);

require_once(INCLUDES_DIR . "/IT.php");

class Page
{
	var $title;
	var $vars;
	var $timer;
	var $cache;
	var $cache_file;

	/**
	 * Constructor
	 *
	 * @param string $title Title of the page (for <title> tag)
	 */
	
	function Page($title=DEFAULT_TITLE)
	{
		if(TIMER===true) {
			$this->timer = $this->getTime();
		}

		$this->title = $title;
		$this->meta = array();
		$this->vars = array();

		ob_start();
	}
	
	/**
	 * Set whether we are caching or now. Can't use the globally defined
	 * FEATURE_CACHE flag as it should only be honored by the image page
	 * handler. Caching is only implemented for sqlite
	 */
	
	function setCache($setting,$path)
	{
		global $db;
		$this->cache = $setting;
		if($db->phptype!="sqlite") {
			$this->cache = false;
		}
		$this->cache_file = $path . "." . $_GET["p"];
	}

	/**
	 * Check whether we have a cache file to deliver. If so, deliver it.
	 * NOTE: this function does not return if it successfully delivered
	 * a cached file
	 */
	
	function checkCache($calling_file,$path)
	{
		global $db;

		if(FEATURE_CACHE===true) {
			if($db->phptype!="sqlite") {
				return;
			}
			$dbfile = $db->dsn["database"];
			$filename = CACHE_DIR . "/" . $this->cache_file;
			if(file_exists($filename)) {
				$cache_time = filemtime($filename);
				$now = date("U");
				$db_time = filemtime($dbfile);
				$age_diff = $cache_time - $db_time;
				$cache_age = $now - $cache_time;
				$caller_time = filemtime($calling_file);
				$stale = FEATURE_CACHE_REFRESH * 60;

				if( $age_diff > 0 and
					$cache_age < $stale and
					$cache_time > $caller_time) {
					readfile($filename);
					if(TIMER===true) {
						$elapsed = $this->timeElapsed();
						echo "<!-- cache read: $elapsed ms -->";
					}
					exit;
				}
			}
		}
	}

	/**
	 * Deal with the meta string. This function only concerns itself
	 * with placing the requested string in the right place, so the string
	 * should be complete. Call multiple times to add multiple strings.
	 *
	 * @param string $meta meta string to add to page
	 */
	
	function meta($meta)
	{
		if ($meta) {
			array_push($this->meta,$meta);
		}
	}

	/**
	 * Set arbitrary template variable (placeholder)
	 * @param string $var Placeholder name
	 * @param string $value Placeholder value
	 */
	function set($var,$value)
	{
		if ($var && $value) {
			$this->vars[$var] = $value;
		}
	}

	/**
	 * Display the page
	 *
	 */
	
	function display()
	{
		$content = ob_get_contents();
		ob_end_clean();

		$modified = date("d F Y H:i:s.", getlastmod());
		$year = date("Y");

		$tmpl = new HTML_Template_IT(TEMPLATE_DIR,array("use_preg"=>false));
		$tmpl->loadTemplateFile(TMPL_MAIN,true,true);

		$tmpl->setVariable("TITLE",$this->title);
		$tmpl->setVariable("CONTENT",$content);

		if (count($this->meta)) {
			$tmpl->setVariable("META",implode("\n",$this->meta));
		}

		// Arbitrary vars...
		$this->vars["ROOT"] = ROOT;
		$this->vars["TZ"] = date("Z");
		foreach($this->vars as $var=>$value) {
			$tmpl->setVariable($var,$value);
		}

		if (FEATURE_SHOWALL===true) {
			$tmpl->setVariable("ROOT_SHOWALL",ROOT);
		}
		if (FEATURE_CHAT===true) {
			$tmpl->setVariable("ROOT_CHAT",ROOT);
		}
		if (FEATURE_RSS===true) {
			$tmpl->setVariable("ROOT_RSS",ROOT);
		}
		if (FEATURE_STYLES_MENU===true) {
			$tmpl->touchBlock("FEATURE_STYLES_MENU");
		}

		$tmpl->setVariable("FOOTER",'<a href="http://ochiba.x-maru.org/">&laquo; Powered By Ochiba &raquo;</a>');

		// Write to the cache. Using flock() here to avoid some possible
		// problems, but see http://www.php.net/manual/en/function.flock.php
		// (in particular, the notes) for discussion on possible race
		// conditions. fclose() releases lock.
		if ($this->cache===true) {
			$filename = CACHE_DIR . "/" . $this->cache_file;
			if($fp = @fopen($filename,"w")) {
				@flock($fp,LOCK_EX);
				@fwrite($fp,$tmpl->get());
				fclose($fp);
			}
		}		

		$tmpl->show();
		
		// Don't put timing in cache - so this comes later
		if (TIMER===true) {
			echo "<!-- generated: " . $this->timeElapsed() . " -->";
        }

	}

	function fail($msg)
	{
		if($msg) {
			echo "<p class=\"alert\">$msg</p>\n";
		}
		$this->display();
		exit;
	}
    function getTime()
	{
		list($usec, $sec) = explode(" ",microtime());
		return ((float)$usec + (float)$sec);
	}

	function timeElapsed()
	{
		$stop = $this->getTime();
		$elapsed = sprintf("%.2f",($stop-$this->timer)*1000);
		return $elapsed;
	}

}
